@@ -12,7 +12,6 @@
 #include "opengl/eglswapchain.h"
 #include "opengl/glrendertimequery.h"
 #include "opengl/glutils.h"
-#include "platformsupport/scenes/opengl/basiceglsurfacetexture_wayland.h"
 #include "utils/softwarevsyncmonitor.h"
 #include "virtual_backend.h"
 #include "virtual_logging.h"
@@ -36,7 +35,7 @@ std::shared_ptr<GLTexture> VirtualEglLayer::texture() const
 
 std::optional<OutputLayerBeginFrameInfo> VirtualEglLayer::doBeginFrame()
 {
-    m_backend->makeCurrent();
+    m_backend->openglContext()->makeCurrent();
 
     const QSize nativeSize = m_output->modeSize();
     if (!m_swapchain || m_swapchain->size() != nativeSize) {
@@ -79,8 +78,7 @@ QHash<uint32_t, QList<uint64_t>> VirtualEglLayer::supportedDrmFormats() const
 }
 
 VirtualEglBackend::VirtualEglBackend(VirtualBackend *b)
-    : AbstractEglBackend()
-    , m_backend(b)
+    : m_backend(b)
 {
 }
 
@@ -139,7 +137,6 @@ void VirtualEglBackend::init()
         return;
     }
 
-    setSupportsBufferAge(false);
     initWayland();
 
     const auto outputs = m_backend->outputs();
@@ -153,34 +150,30 @@ void VirtualEglBackend::init()
 
 bool VirtualEglBackend::initRenderingContext()
 {
-    return createContext(EGL_NO_CONFIG_KHR) && makeCurrent();
+    return createContext(EGL_NO_CONFIG_KHR) && openglContext()->makeCurrent();
 }
 
 void VirtualEglBackend::addOutput(Output *output)
 {
-    makeCurrent();
+    openglContext()->makeCurrent();
     m_outputs[output] = std::make_unique<VirtualEglLayer>(output, this);
 }
 
 void VirtualEglBackend::removeOutput(Output *output)
 {
-    makeCurrent();
+    openglContext()->makeCurrent();
     m_outputs.erase(output);
 }
 
-std::unique_ptr<SurfaceTexture> VirtualEglBackend::createSurfaceTextureWayland(SurfacePixmap *pixmap)
-{
-    return std::make_unique<BasicEGLSurfaceTextureWayland>(this, pixmap);
-}
-
 OutputLayer *VirtualEglBackend::primaryLayer(Output *output)
 {
     return m_outputs[output].get();
 }
 
-void VirtualEglBackend::present(Output *output, const std::shared_ptr<OutputFrame> &frame)
+bool VirtualEglBackend::present(Output *output, const std::shared_ptr<OutputFrame> &frame)
 {
     static_cast<VirtualOutput *>(output)->present(frame);
+    return true;
 }
 
 std::pair<std::shared_ptr<KWin::GLTexture>, ColorDescription> VirtualEglBackend::textureForOutput(Output *output) const
