@@ -20,6 +20,7 @@
 
 #if KWIN_BUILD_X11
 #include "utils/xcbutils.h"
+#include <xcb/xcb_cursor.h>
 #endif
 // KDE
 #include <KConfig>
@@ -30,8 +31,6 @@
 #include <QScreen>
 #include <QTimer>
 
-#include <xcb/xcb_cursor.h>
-
 namespace KWin
 {
 Cursors *Cursors::s_self = nullptr;
@@ -43,32 +42,35 @@ Cursors *Cursors::self()
     return s_self;
 }
 
+Cursors::Cursors()
+    : m_mouse(std::make_unique<Cursor>())
+{
+    addCursor(m_mouse.get());
+    setCurrentCursor(m_mouse.get());
+}
+
 void Cursors::addCursor(Cursor *cursor)
 {
     Q_ASSERT(!m_cursors.contains(cursor));
     m_cursors += cursor;
 
+    connect(cursor, &Cursor::destroyed, this, [this, cursor]() {
+        m_cursors.removeOne(cursor);
+        if (m_currentCursor == cursor) {
+            if (m_cursors.isEmpty()) {
+                m_currentCursor = nullptr;
+            } else {
+                setCurrentCursor(m_cursors.constFirst());
+            }
+        }
+    });
+
     connect(cursor, &Cursor::posChanged, this, [this, cursor](const QPointF &pos) {
         setCurrentCursor(cursor);
         Q_EMIT positionChanged(cursor, pos);
     });
 }
 
-void Cursors::removeCursor(Cursor *cursor)
-{
-    m_cursors.removeOne(cursor);
-    if (m_currentCursor == cursor) {
-        if (m_cursors.isEmpty()) {
-            m_currentCursor = nullptr;
-        } else {
-            setCurrentCursor(m_cursors.constFirst());
-        }
-    }
-    if (m_mouse == cursor) {
-        m_mouse = nullptr;
-    }
-}
-
 void Cursors::hideCursor()
 {
     m_cursorHideCounter++;
@@ -120,16 +122,9 @@ Cursor::Cursor()
     QDBusConnection::sessionBus().connect(QString(), QStringLiteral("/KGlobalSettings"), QStringLiteral("org.kde.KGlobalSettings"),
                                           QStringLiteral("notifyChange"), this, SLOT(slotKGlobalSettingsNotifyChange(int, int)));
 
-    if (kwinApp()->operationMode() != Application::OperationModeWaylandOnly) {
-        connect(kwinApp(), &Application::x11ConnectionChanged, this, [this]() {
-            m_cursors.clear();
-        });
-    }
-}
-
-Cursor::~Cursor()
-{
-    Cursors::self()->removeCursor(this);
+    connect(kwinApp(), &Application::x11ConnectionChanged, this, [this]() {
+        m_cursors.clear();
+    });
 }
 
 void Cursor::loadThemeSettings()
@@ -204,18 +199,16 @@ QRectF Cursor::rect() const
 
 QPointF Cursor::pos()
 {
-    doGetPos();
     return m_pos;
 }
 
 void Cursor::setPos(const QPointF &pos)
 {
-    // first query the current pos to not warp to the already existing pos
-    if (pos == m_pos) {
+    if (m_pos == pos) {
         return;
     }
     m_pos = pos;
-    doSetPos();
+    Q_EMIT posChanged(m_pos);
 }
 
 #if KWIN_BUILD_X11
@@ -260,24 +253,6 @@ xcb_cursor_t Cursor::x11Cursor(const QByteArray &name)
 }
 #endif
 
-void Cursor::doSetPos()
-{
-    Q_EMIT posChanged(m_pos);
-}
-
-void Cursor::doGetPos()
-{
-}
-
-void Cursor::updatePos(const QPointF &pos)
-{
-    if (m_pos == pos) {
-        return;
-    }
-    m_pos = pos;
-    Q_EMIT posChanged(m_pos);
-}
-
 QString Cursor::defaultThemeName()
 {
     return QStringLiteral("default");
@@ -574,12 +549,37 @@ QList<QByteArray> CursorShape::alternatives(const QByteArray &name)
                 QByteArrayLiteral("right_side"),
             },
         },
+        {
+            QByteArrayLiteral("dnd-ask"),
+            {
+                QByteArrayLiteral("copy"),
+            },
+        },
+        {
+            QByteArrayLiteral("all-resize"),
+            {
+                QByteArrayLiteral("move"),
+            },
+        },
     };
+
     auto it = alternatives.find(name);
-    if (it != alternatives.end()) {
-        return it.value();
+    if (it == alternatives.end()) {
+        return QList<QByteArray>();
     }
-    return QList<QByteArray>();
+
+    QList<QByteArray> result = it.value();
+    for (int i = 0; i < result.size(); ++i) {
+        if (auto it = alternatives.find(result[i]); it != alternatives.end()) {
+            for (const QByteArray &alternative : *it) {
+                if (!result.contains(alternative)) {
+                    result.append(alternative);
+                }
+            }
+        }
+    }
+
+    return result;
 }
 
 QByteArray CursorShape::name() const
