@@ -11,9 +11,13 @@
 
 #pragma once
 
+#include <QStandardItemModel>
+
 #include <kcmodule.h>
 #include <ksharedconfig.h>
 
+class QProcess;
+
 namespace KWin
 {
 class KWinTabBoxConfigForm;
@@ -47,9 +51,12 @@ private:
 private:
     KWinTabBoxConfigForm *m_primaryTabBoxUi = nullptr;
     KWinTabBoxConfigForm *m_alternativeTabBoxUi = nullptr;
+    std::unique_ptr<QStandardItemModel> m_switcherModel;
     KSharedConfigPtr m_config;
 
     TabBox::KWinTabboxData *m_data;
+
+    std::unique_ptr<QProcess> m_previewProcess;
 };
 
 } // namespace
