@@ -110,7 +110,6 @@ GLTexturePrivate::GLTexturePrivate()
     , m_filter(GL_NEAREST)
     , m_wrapMode(GL_REPEAT)
     , m_canUseMipmaps(false)
-    , m_markedDirty(false)
     , m_filterChanged(true)
     , m_wrapModeChanged(false)
     , m_owning(true)
@@ -122,6 +121,10 @@ GLTexturePrivate::GLTexturePrivate()
 
 GLTexturePrivate::~GLTexturePrivate()
 {
+    if (!EglContext::currentContext()) {
+        qCWarning(KWIN_OPENGL, "Could not delete texture because no context is current");
+        return;
+    }
     if (m_texture != 0 && m_owning) {
         glDeleteTextures(1, &m_texture);
     }
@@ -154,7 +157,7 @@ void GLTexture::update(const QImage &image, const QRegion &region, const QPoint
 
     Q_ASSERT(d->m_owning);
 
-    const auto context = OpenGlContext::currentContext();
+    const auto context = EglContext::currentContext();
     GLenum glFormat;
     GLenum type;
     QImage::Format uploadFormat;
@@ -211,9 +214,6 @@ void GLTexture::bind()
 
     glBindTexture(d->m_target, d->m_texture);
 
-    if (d->m_markedDirty) {
-        onDamage();
-    }
     if (d->m_filterChanged) {
         GLenum minFilter = GL_NEAREST;
         GLenum magFilter = GL_NEAREST;
@@ -353,11 +353,6 @@ GLenum GLTexture::internalFormat() const
     return d->m_internalFormat;
 }
 
-bool GLTexture::isDirty() const
-{
-    return d->m_markedDirty;
-}
-
 void GLTexture::setFilter(GLenum filter)
 {
     if (filter != d->m_filter) {
@@ -374,15 +369,6 @@ void GLTexture::setWrapMode(GLenum mode)
     }
 }
 
-void GLTexture::setDirty()
-{
-    d->m_markedDirty = true;
-}
-
-void GLTexture::onDamage()
-{
-}
-
 void GLTexturePrivate::updateMatrix()
 {
     const QMatrix4x4 textureToBufferMatrix = m_textureToBufferTransform.toMatrix();
@@ -423,7 +409,7 @@ OutputTransform GLTexture::contentTransform() const
 
 void GLTexture::setSwizzle(GLenum red, GLenum green, GLenum blue, GLenum alpha)
 {
-    if (!OpenGlContext::currentContext()->isOpenGLES()) {
+    if (!EglContext::currentContext()->isOpenGLES()) {
         const GLuint swizzle[] = {red, green, blue, alpha};
         glTexParameteriv(d->m_target, GL_TEXTURE_SWIZZLE_RGBA, (const GLint *)swizzle);
     } else {
@@ -451,12 +437,12 @@ QMatrix4x4 GLTexture::matrix(TextureCoordinateType type) const
 
 bool GLTexture::supportsSwizzle()
 {
-    return OpenGlContext::currentContext()->supportsTextureSwizzle();
+    return EglContext::currentContext()->supportsTextureSwizzle();
 }
 
 bool GLTexture::supportsFormatRG()
 {
-    return OpenGlContext::currentContext()->supportsRGTextures();
+    return EglContext::currentContext()->supportsRGTextures();
 }
 
 QImage GLTexture::toImage()
@@ -466,7 +452,7 @@ QImage GLTexture::toImage()
     }
     QImage ret(size(), QImage::Format_RGBA8888_Premultiplied);
 
-    if (OpenGlContext::currentContext()->isOpenGLES()) {
+    if (EglContext::currentContext()->isOpenGLES()) {
         GLFramebuffer fbo(this);
         GLFramebuffer::pushFramebuffer(&fbo);
         glReadPixels(0, 0, width(), height(), GL_RGBA, GL_UNSIGNED_INT_8_8_8_8_REV, ret.bits());
@@ -500,7 +486,7 @@ std::unique_ptr<GLTexture> GLTexture::allocate(GLenum internalFormat, const QSiz
     }
     glBindTexture(GL_TEXTURE_2D, texture);
 
-    const auto context = OpenGlContext::currentContext();
+    const auto context = EglContext::currentContext();
     if (!context->isOpenGLES()) {
         if (context->supportsTextureStorage()) {
             glTexStorage2D(GL_TEXTURE_2D, levels, internalFormat, size.width(), size.height());
@@ -536,7 +522,7 @@ std::unique_ptr<GLTexture> GLTexture::upload(const QImage &image)
         return nullptr;
     }
 
-    const auto context = OpenGlContext::currentContext();
+    const auto context = EglContext::currentContext();
     GLenum internalFormat;
     GLenum format;
     GLenum type;
