@@ -45,7 +45,7 @@ FrogColorManagementSurfaceV1::~FrogColorManagementSurfaceV1()
     if (m_surface) {
         const auto priv = SurfaceInterfacePrivate::get(m_surface);
         priv->pending->colorDescription = ColorDescription::sRGB;
-        priv->pending->colorDescriptionIsSet = true;
+        priv->pending->committed |= SurfaceState::Field::ColorDescription;
         priv->frogColorManagement = nullptr;
     }
 }
@@ -110,10 +110,10 @@ void FrogColorManagementSurfaceV1::frog_color_managed_surface_set_known_containe
     switch (primaries) {
     case primaries_undefined:
     case primaries_rec709:
-        m_containerColorimetry = NamedColorimetry::BT709;
+        m_containerColorimetry = Colorimetry::BT709;
         break;
     case primaries_rec2020:
-        m_containerColorimetry = NamedColorimetry::BT2020;
+        m_containerColorimetry = Colorimetry::BT2020;
         break;
     }
     updateColorDescription();
@@ -197,9 +197,9 @@ void FrogColorManagementSurfaceV1::updateColorDescription()
             m_maxAverageLuminance,
             m_maxPeakBrightness,
             m_masteringColorimetry,
-            Colorimetry::fromName(NamedColorimetry::BT709),
+            Colorimetry::BT709,
         };
-        priv->pending->colorDescriptionIsSet = true;
+        priv->pending->committed |= SurfaceState::Field::ColorDescription;
     }
 }
 
