@@ -383,13 +383,13 @@ void XdgToplevelInterfacePrivate::reset()
     auto xdgSurfacePrivate = XdgSurfaceInterfacePrivate::get(xdgSurface);
     xdgSurfacePrivate->reset();
 
-    windowTitle = QString();
-    windowClass = QString();
+    title = QString();
+    appId = QString();
+    tag = QString();
+    description = QString();
     minimumSize = QSize(0, 0);
     maximumSize = QSize(0, 0);
     customIcon = QIcon();
-    pending = XdgToplevelCommit{};
-    stashed.clear();
 
     Q_EMIT q->resetOccurred();
 }
@@ -416,20 +416,20 @@ void XdgToplevelInterfacePrivate::xdg_toplevel_set_parent(Resource *resource, ::
 
 void XdgToplevelInterfacePrivate::xdg_toplevel_set_title(Resource *resource, const QString &title)
 {
-    if (windowTitle == title) {
+    if (this->title == title) {
         return;
     }
-    windowTitle = title;
-    Q_EMIT q->windowTitleChanged(title);
+    this->title = title;
+    Q_EMIT q->titleChanged(title);
 }
 
 void XdgToplevelInterfacePrivate::xdg_toplevel_set_app_id(Resource *resource, const QString &app_id)
 {
-    if (windowClass == app_id) {
+    if (appId == app_id) {
         return;
     }
-    windowClass = app_id;
-    Q_EMIT q->windowClassChanged(app_id);
+    appId = app_id;
+    Q_EMIT q->appIdChanged(app_id);
 }
 
 void XdgToplevelInterfacePrivate::xdg_toplevel_show_window_menu(Resource *resource, ::wl_resource *seatResource, uint32_t serial, int32_t x, int32_t y)
@@ -467,8 +467,42 @@ void XdgToplevelInterfacePrivate::xdg_toplevel_resize(Resource *resource, ::wl_r
         return;
     }
 
+    Gravity gravity;
+    switch (xdgEdges) {
+    case resize_edge_none:
+        gravity = Gravity::None;
+        break;
+    case resize_edge_top:
+        gravity = Gravity::Top;
+        break;
+    case resize_edge_bottom:
+        gravity = Gravity::Bottom;
+        break;
+    case resize_edge_left:
+        gravity = Gravity::Left;
+        break;
+    case resize_edge_top_left:
+        gravity = Gravity::TopLeft;
+        break;
+    case resize_edge_bottom_left:
+        gravity = Gravity::BottomLeft;
+        break;
+    case resize_edge_right:
+        gravity = Gravity::Right;
+        break;
+    case resize_edge_top_right:
+        gravity = Gravity::TopRight;
+        break;
+    case resize_edge_bottom_right:
+        gravity = Gravity::BottomRight;
+        break;
+    default:
+        wl_resource_post_error(resource->handle, error_invalid_resize_edge, "invalid resize edge");
+        return;
+    }
+
     SeatInterface *seat = SeatInterface::get(seatResource);
-    Q_EMIT q->resizeRequested(seat, XdgToplevelInterface::ResizeAnchor(xdgEdges), serial);
+    Q_EMIT q->resizeRequested(seat, gravity, serial);
 }
 
 void XdgToplevelInterfacePrivate::xdg_toplevel_set_max_size(Resource *resource, int32_t width, int32_t height)
@@ -477,7 +511,7 @@ void XdgToplevelInterfacePrivate::xdg_toplevel_set_max_size(Resource *resource,
         wl_resource_post_error(resource->handle, error_invalid_size, "width and height must be positive or zero");
         return;
     }
-    pending.maximumSize = QSize(width, height);
+    pending->maximumSize = QSize(width, height);
 }
 
 void XdgToplevelInterfacePrivate::xdg_toplevel_set_min_size(Resource *resource, int32_t width, int32_t height)
@@ -486,7 +520,7 @@ void XdgToplevelInterfacePrivate::xdg_toplevel_set_min_size(Resource *resource,
         wl_resource_post_error(resource->handle, error_invalid_size, "width and height must be positive or zero");
         return;
     }
-    pending.minimumSize = QSize(width, height);
+    pending->minimumSize = QSize(width, height);
 }
 
 void XdgToplevelInterfacePrivate::xdg_toplevel_set_maximized(Resource *resource)
@@ -530,7 +564,7 @@ XdgToplevelInterface::XdgToplevelInterface(XdgSurfaceInterface *xdgSurface, ::wl
 {
     XdgSurfaceInterfacePrivate *surfacePrivate = XdgSurfaceInterfacePrivate::get(xdgSurface);
     surfacePrivate->toplevel = this;
-    surfacePrivate->pending = &d->pending;
+    surfacePrivate->pending = d->pending;
 
     d->init(resource);
 }
@@ -575,14 +609,24 @@ XdgToplevelInterface *XdgToplevelInterface::parentXdgToplevel() const
     return d->parentXdgToplevel;
 }
 
-QString XdgToplevelInterface::windowTitle() const
+QString XdgToplevelInterface::title() const
 {
-    return d->windowTitle;
+    return d->title;
 }
 
-QString XdgToplevelInterface::windowClass() const
+QString XdgToplevelInterface::appId() const
 {
-    return d->windowClass;
+    return d->appId;
+}
+
+QString XdgToplevelInterface::tag() const
+{
+    return d->tag;
+}
+
+QString XdgToplevelInterface::description() const
+{
+    return d->description;
 }
 
 QSize XdgToplevelInterface::minimumSize() const
@@ -600,6 +644,11 @@ QIcon XdgToplevelInterface::customIcon() const
     return d->customIcon;
 }
 
+XdgToplevelSessionV1Interface *XdgToplevelInterface::session() const
+{
+    return d->session;
+}
+
 quint32 XdgToplevelInterface::sendConfigure(const QSize &size, const States &states)
 {
     // Note that the states listed in the configure event must be an array of uint32_t.
@@ -699,6 +748,11 @@ XdgToplevelInterface *XdgToplevelInterface::get(::wl_resource *resource)
     return nullptr;
 }
 
+wl_resource *XdgToplevelInterface::resource() const
+{
+    return d->resource()->handle;
+}
+
 XdgPopupInterfacePrivate *XdgPopupInterfacePrivate::get(XdgPopupInterface *popup)
 {
     return popup->d.get();
@@ -745,8 +799,6 @@ void XdgPopupInterfacePrivate::apply(XdgPopupCommit *commit)
 void XdgPopupInterfacePrivate::reset()
 {
     auto xdgSurfacePrivate = XdgSurfaceInterfacePrivate::get(xdgSurface);
-    pending = XdgPopupCommit{};
-    stashed.clear();
     xdgSurfacePrivate->reset();
 }
 
@@ -784,7 +836,7 @@ XdgPopupInterface::XdgPopupInterface(XdgSurfaceInterface *xdgSurface, SurfaceInt
 {
     XdgSurfaceInterfacePrivate *surfacePrivate = XdgSurfaceInterfacePrivate::get(xdgSurface);
     surfacePrivate->popup = this;
-    surfacePrivate->pending = &d->pending;
+    surfacePrivate->pending = d->pending;
 
     d->parentSurface = parentSurface;
     d->positioner = positioner;
